@@ -5,14 +5,14 @@
 #
 
 package PDL::PP::Code;
-use Carp;
-our @CARP_NOT;
 
 use strict;
+use warnings;
+use Carp;
 
 sub get_pdls {my($this) = @_; return ($this->{ParNames},$this->{ParObjs});}
 
-my @code_args_always = qw(SignatureObj GenericTypes ExtraGenericLoops HaveThreading Name);
+my @code_args_always = qw(SignatureObj GenericTypes ExtraGenericSwitches HaveThreading Name);
 sub make_args {
   my ($which) = @_;
   ("Parsed$which", [$which,\"Bad$which",@code_args_always]);
@@ -129,7 +129,7 @@ sub new {
 
     # Enclose it all in a genericloop.
     my $nc = $coderef;
-    $coderef = PDL::PP::GenericLoop->new($generictypes, undef,
+    $coderef = PDL::PP::GenericSwitch->new($generictypes, undef,
 	  [grep {!$extrageneric->{$_}} @$parnames],'$PRIV(__datatype)');
     push @{$coderef},$nc;
 
@@ -142,7 +142,7 @@ sub new {
     my $no = 0;
     for(sort keys %glh) {
 	my $nc = $coderef;
-	$coderef = PDL::PP::GenericLoop->new($generictypes,$no++,
+	$coderef = PDL::PP::GenericSwitch->new($generictypes,$no++,
 					    $glh{$_},$_);
 	push @$coderef,$nc;
     }
@@ -389,24 +389,19 @@ sub report_error {
 # 	new - constructor
 #	get_str - get the string to be put into the xsub.
 
-###########################
-#
-# Encapsulate a block
-
 package PDL::PP::Block;
 
 sub new { my($type) = @_; bless [],$type; }
 
 sub myoffs { 0 }
 sub myprelude {}
-sub myitem { "" }
 sub mypostlude {}
 
 sub get_str {
     my ($this,$parent,$context) = @_;
     my $str = $this->myprelude($parent,$context);
-    $str .= $this->get_str_int($parent,$context);
-    $str .= $this->mypostlude($parent,$context);
+    $str .= $this->get_str_int($parent,$context)//'';
+    $str .= $this->mypostlude($parent,$context)//'';
     return $str;
 }
 
@@ -415,10 +410,11 @@ sub get_str_int {
   my $nth=0;
   my $str = "";
   MYLOOP: while(1) {
-    my $it = $this->myitem($parent,$nth);
+    my $it = $this->can('myitemstart') && $this->myitemstart($parent,$nth);
     last MYLOOP if $nth and !$it;
-    $str .= $it;
+    $str .= $it//'';
     $str .= join '', $this->get_contained($parent,$context);
+    $str .= $it if $it = $this->can('myitemend') && $this->myitemend($parent,$nth);
     $nth++;
   }
   return $str;
@@ -459,10 +455,6 @@ if ( \$PRIV(bvalflag) ) { PDL_COMMENT("** do 'bad' Code **")
 EOF
 }
 
-###########################
-#
-# Encapsulate a loop
-
 package PDL::PP::Loop;
 our @ISA = "PDL::PP::Block";
 
@@ -498,10 +490,7 @@ sub mypostlude { my($this,$parent,$context) = @_;
 	return join '', map PDL::PP::pp_line_numbers(__LINE__-1, "}} PDL_COMMENT(\"Close $_\")"), @{$this->[0]};
 }
 
-###########################
-#
-# Encapsulate a generic type loop
-package PDL::PP::GenericLoop;
+package PDL::PP::GenericSwitch;
 our @ISA = "PDL::PP::Block";
 
 # make the typetable from info in PDL::Types
@@ -524,17 +513,14 @@ sub myoffs {4}
 sub myprelude {
     my ($this,$parent,$context) = @_;
     push @{$parent->{Gencurtype}}, undef; # so that $GENERIC can get at it
-    die "ERROR: need to rethink NaN support in GenericLoop\n"
+    die "ERROR: need to rethink NaN support in GenericSwitch\n"
 	if defined $this->[1] and $parent->{ftypes_type};
-    <<WARNING_EATER;
-PDL_COMMENT("Start generic loop")
-	switch($this->[3]) { case -42: PDL_COMMENT("Warning eater") {(void)1;
-WARNING_EATER
+    qq[PDL_COMMENT("Start generic loop")\n\tswitch($this->[3]) {\n];
 }
 
-sub myitem {
+sub myitemstart {
     my ($this,$parent,$nth) = @_;
-    my $item = $this->[0]->[$nth] || return "";
+    my $item = $this->[0][$nth] || return "";
     $parent->{Gencurtype}[-1] = $item;
     @$parent{qw(ftypes_type ftypes_vars)} = ($item, $this->[2]) if defined $this->[1];
     my ($ord,$pdls) = $parent->get_pdls;
@@ -544,17 +530,22 @@ sub myitem {
       : join '', map $_->get_xsdatapdecl($_->adjusted_type($item)->ctype),
           map $parent->{ParObjs}{$_}, sort keys %{$this->[2]};
     join '',
-	PDL::PP::pp_line_numbers(__LINE__-1, "\t} break; case @{[$item->sym]}: {\n"),
+	PDL::PP::pp_line_numbers(__LINE__-1, "case @{[$item->sym]}: {\n"),
 	$decls;
 }
 
+sub myitemend {
+    my ($this,$parent,$nth) = @_;
+    my $item = $this->[0][$nth] || return "";
+    PDL::PP::pp_line_numbers(__LINE__-1, "} break;\n");
+}
+
 sub mypostlude {
     my($this,$parent,$context) = @_;
     pop @{$parent->{Gencurtype}};  # and clean up the Gentype stack
     $parent->{ftypes_type} = undef if defined $this->[1];
     my $supported = join '', map $_->ppsym, @{$this->[0]};
-    "\tbreak;}
-	default:return PDL->make_error(PDL_EUSERERROR, \"PP INTERNAL ERROR in $parent->{Name}: unhandled datatype(%d), only handles ($supported)! PLEASE MAKE A BUG REPORT\\n\", $this->[3]);}\n";
+    "\n\tdefault:return PDL->make_error(PDL_EUSERERROR, \"PP INTERNAL ERROR in $parent->{Name}: unhandled datatype(%d), only handles ($supported)! PLEASE MAKE A BUG REPORT\\n\", $this->[3]);}\n";
 }
 
 ####
@@ -565,7 +556,6 @@ sub mypostlude {
 package PDL::PP::ThreadLoop;
 use Carp;
 our @ISA = "PDL::PP::Block";
-our @CARP_NOT;
 
 sub new {
 	my $type   = shift;
@@ -588,7 +578,6 @@ sub mypostlude {my($this,$parent,$context) = @_;
 package PDL::PP::BackCodeThreadLoop;
 use Carp;
 our @ISA = "PDL::PP::ThreadLoop";
-our @CARP_NOT;
 
 sub myprelude {
     my($this,$parent,$context, $backcode) = @_;
@@ -609,7 +598,6 @@ package PDL::PP::Types;
 use Carp;
 use PDL::Types ':All';
 our @ISA = "PDL::PP::Block";
-our @CARP_NOT;
 my %types = map +($_=>1), ppdefs_all; # BSUL....
 
 sub new {
@@ -628,13 +616,8 @@ sub get_str {
 }
 
 
-###########################
-#
-# Encapsulate an access
-
 package PDL::PP::Access;
 use Carp;
-our @CARP_NOT;
 
 sub new { my($type,$pdl,$inds,$parent) = @_;
     bless [$pdl,$inds],$type;
@@ -646,10 +629,8 @@ sub get_str { my($this,$parent,$context) = @_;
 }
 
 ###########################
-#
 # Encapsulate a check on whether a value is good or bad
 # handles both checking (good/bad) and setting (bad)
-
 package PDL::PP::BadAccess;
 use Carp;
 
@@ -686,19 +667,17 @@ sub get_str {
     my $lhs = $getters{$get}->($obj, $inds, $context);
     my $rhs = "${name}_badval";
     print "DBG:  [$lhs $op $rhs]\n" if $::PP_VERBOSE;
-    $op->($lhs, $rhs, $obj->adjusted_type($parent->{Gencurtype}[-1])->ppsym);
+    my $type = exists $parent->{ftypes_vars}{$name}
+	? $parent->{ftypes_type}
+	: $obj->adjusted_type($parent->{Gencurtype}[-1]);
+    $op->($lhs, $rhs, $type->ppsym);
 }
 
 
-###########################
-#
-# Encapsulate a macroaccess
-
 package PDL::PP::MacroAccess;
 use Carp;
 use PDL::Types ':All';
 my $types = join '',ppdefs_all;
-our @CARP_NOT;
 
 sub new {
     my ($type, $pdl, $inds, $gentypes, $name) = @_;
@@ -708,7 +687,7 @@ sub new {
     my @lst = split ',', $inds, -1;
     confess "Macroaccess: different nos of args $pdl $inds\n" if @lst != @ilst;
     my %type2value; @type2value{@ilst} = @lst;
-    warn "$name has no Macro for generic type $_ (has $pdl)\n"
+    confess "$name has no Macro for generic type $_ (has $pdl)\n"
 	for grep !exists $type2value{$_}, @$gentypes;
     my %gts; @gts{@$gentypes} = ();
     warn "Macro for unsupported generic type identifier $_\n"
@@ -724,14 +703,8 @@ sub get_str {
     $type2value->{$parent->{Gencurtype}[-1]->ppsym};
 }
 
-
-###########################
-#
-# Encapsulate a GentypeAccess
-
 package PDL::PP::GentypeAccess;
 use Carp;
-our @CARP_NOT;
 
 sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
 
@@ -745,7 +718,6 @@ sub get_str {my($this,$parent,$context) = @_;
 
 package PDL::PP::PpsymAccess;
 use Carp;
-our @CARP_NOT;
 
 sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
 
